import unittest, time
from MemcachedGenericCache import MemcachedGenericCache
import decorators

class TestSimple(unittest.TestCase):
    
    def setUp(self):
        self.cache = MemcachedGenericCache(["127.0.0.1:11211"])
        self.cache.clear()

    def testempty(self):
        self.assertEqual(self.cache["invalid"], None)
        self.assertRaises(KeyError, self.cache.fetch, "invalid", True)

    def testinsert(self):
        self.cache["value"] = 42
        self.assertEqual(self.cache["value"], 42)

    def testmultidel(self):
        self.cache["value1"] = 42
        self.cache["value2"] = 43
        self.cache["value3"] = 44
        self.cache.delete_multi([ 'value1', 'value2', 'value4' ])
        self.assertEqual(self.cache["value3"], 44)
        self.assertEqual(self.cache["value2"], None)
        self.assertEqual(self.cache["value1"], None)

class TestClear(unittest.TestCase):
    
    def setUp(self):
        self.cache = MemcachedGenericCache(["127.0.0.1:11211"])
        self.cache.clear()

    def testclear(self):
        for i in range(10):
            self.cache[i] = i + 42
        self.cache.clear()
        self.assertEqual(self.cache[3], None)
        self.cache[3] = 42
        self.assertEqual(self.cache[3], 42)
       
        
class TestTimeout(unittest.TestCase):
    
    def setUp(self):
        self.cache = MemcachedGenericCache(["127.0.0.1:11211"], expiry = 3)
        self.cache.clear()

    def testtimeout(self):
        # We fill the cache with three values
        self.cache["1"] = 42
        self.cache["2"] = 43
        self.cache["3"] = 44
        self.cache["4"] = 45
        self.cache["5"] = 46
        # We check the values are in
        self.assertEqual(self.cache["1"], 42)
        self.assertEqual(self.cache["2"], 43)
        self.assertEqual(self.cache["3"], 44)
        self.assertEqual(self.cache["4"], 45)
        self.assertEqual(self.cache["5"], 46)
        # We sleep 2 seconds
        time.sleep(2)
        # We check the values are still in
        self.assertEqual(self.cache["1"], 42)
        self.assertEqual(self.cache["2"], 43)
        self.assertEqual(self.cache["3"], 44)
        self.assertEqual(self.cache["4"], 45)
        # We refresh the value 2
        self.cache["2"] = 43
        # We sleep 1.5 more seconds
        time.sleep(1.5)
        # We check 2 is still in
        self.assertEqual(self.cache["2"], 43)
        # We check 1 and 4 expired
        self.assertEqual(self.cache["1"], None)
        self.assertEqual(self.cache["4"], None)

global_cache = MemcachedGenericCache(["127.0.0.1:11211"])

class TestDecorators(unittest.TestCase):
    def setUp(self):
        self.nbcall = 0
        global_cache.clear()

    @decorators.cached(global_cache)
    def double(self, what):
        self.nbcall += 1
        return 2 * what

    def testdecorators(self):
        # Fill cache
        self.assertEqual(self.double(2), 4)
        self.assertEqual(self.double("foo"), "foofoo")
        # Size should be 2, and nbcall too
        self.assertEqual(self.nbcall, 2)
        # Read cache
        self.assertEqual(self.double("foo"), "foofoo")
        self.assertEqual(self.double(2), 4)
        # Size should still be 2, and nbcall too
        self.assertEqual(self.nbcall, 2)
        


if __name__ == '__main__':
    unittest.main()
